
about = "The Virtual Screening workflows version 0.1 provide tools to run \n"
about = about + "AutodDck-based virtual screening. Available workflows:\n"
about = about + "  - AutoDockML - run virtual screening for sinlge receptor and multiple ligands using AutoDock\n"
about = about + "  - AutoDockVSlocal - virtual screening with AutoDock and user-provided ligand libraries \n"
about = about + "  - AutoDockVSpublic - virtual screening with AutoDock and ligand libraries on the server\n"
about = about + "  - VinaVSlocal - virtual screening using AutoDock Vina and user-provided ligand libraries\n"
about = about + "  - VinaVSpublic - virtual screening using AutoDock Vina and ligand libraries on the server\n"
